==> linuxconf-tools-1.6.src.tar.gz: linuxconf-tools-1.6/vfs/filedir1.cc <==
#include <stdlib.h>
#include <string.h>
#include "vfs.h"

static int filedir_match (
	FILEDIR *fdir,
	const char *wild,	/* Wild card sans path */
	char *tb[],
	int maxtb,			/* Nombre maximum d'item dans la table */
	const char *addpath,/* Path a inserer devant chaque nom de fichier */
	int matchdir)		/* Recherche les sous-rpertoires */
{
	int nbtb = 0;
	//printf ("avant setup\n"); fflush (stdout);
	//atoi("1"); fflush (stdout);
	filedir_setup (fdir);
	//printf ("apres setup\n"); fflush (stdout);
	{
		WILD w;
		VFILE_SPEC *vf = fdir->incl_files;
		int nb = fdir->fincl_nb;
		int i;
		wild_set (wild,&w);
		for (i=0; i<nb; i++, vf++){
			const char *fname = vf->fname;
			// Le makefile.dat est toujours cach, voir spec plus loin.
			if (strcmp(fname,"makefile.dat")!=0){
				/*
					Certaines entres d'un makefile.dat dcrivent des
					lments d'un sous-rpertoire. On doit les ignors.
				*/
				const char *ptsep = strchr(fname,'/');
				int selectd;
				char fname_tmp[PATH_MAX];
				if (ptsep == NULL) ptsep = strchr(fname,'\\');
				selectd = (ptsep != NULL) + matchdir;
				if (selectd == 2){
					char tmp = *ptsep;
					*(char*)ptsep = '\0';
					strcpy (fname_tmp,fname);
					*(char*)ptsep = tmp;
					fname = fname_tmp;
				}
				if (selectd != 1 && wild_test(fname,&w)){
					if (nbtb < maxtb){
						/*
							Les fonctions dir_getlist_ retourne des noms
							de fichier avec le path original inclus.
						*/
						char fpath[PATH_MAX];
						if (addpath[0] != '\0'){
							path_make (addpath,fname,fpath);
							fname = fpath;
						}
						tb[nbtb++] = strdup (fname);
					}
				}
			}
		}
	}
	return nbtb;
}

/*
	fusionne tbreel dans tb en evitant les rptitions.
	Les deux tables sont tries.

	Retourne le nombre d'item dans tb[]. Les lments de tbreel[] sont
	librs.
*/
static int vdir_fusion (
	char *tb[],
	int nbtb,
	char *tbreel[],
	int nbreel)
{
	int notb = 0;
	int i;
	for (i=0; i<nbreel; i++){
		char *fname = tbreel[i];
		if (strcmp(fname,"makefile.dat")!=0
			&& strcmp(fname,"makefile.del")!=0){
			/* #Spcification: vdir / fichiers de controle / exclusion
				Les fichiers de controle makefile.dat et makefile.del
				sont cachs au fonction usager qui balaye des
				rpertoire: vdir_getlist.. 
			*/
			// makefile.dat est exclu lors du balayage du makefile.dat
			// lui-mme.
			int ajout = 1;		/* Ajoute  la fin de la liste */
			for ( ; notb < nbtb; notb++){
				int cmp = strcmp(fname,tb[notb]);
				if(cmp == 0){
					/* Dj l */
					ajout = 0;
					break;	
				}else if (cmp < 0){
					/* Pas l, on ajoute */
					tbstr_insert (tb,nbtb,fname,notb,0);
					fname = NULL;
					nbtb++;
					notb++;
					ajout = 0;
					break;
				}
			}
			if (ajout){
				tb[nbtb++] = fname;
				fname = NULL;
				notb++;
			}
		}
		free (fname);
	}
	return nbtb;
}
/*
	Obtient la liste des des fichiers dans un rpertoire virtuel
	retourne -1 si erreur, le nombre de fichiers placs dans tb[].
	Les fichiers sont placs via strdup_err(). L'appelant est responsable
	de librer.
*/
int filedir_getlist (
	FILEDIR *fdir,
	const char *wild,		/* Wild card sans path */
	char *tb[],
	int maxtb,				/* Nombre maximum d'item dans la table */
	const char *addpath)	/* path a inserer devant chaque fichiers */
{
//printf ("avant match :%s: :%s: %lx %p\n",wild,addpath,_ESP,filedir_match); fflush (stdout);
	int nbtb = filedir_match (fdir,wild,tb,maxtb,addpath,0);
	char *tbreel[1000];
//printf ("Avant dir_getlist :%s: :%s:\n",addpath,wild);
	int nbreel = dir_getlist (addpath,wild,0,tbreel,1000);
//printf ("nbreel = %d\n",nbreel); fflush (stdout);
	nbtb = vdir_fusion (tb,nbtb,tbreel,nbreel);
//printf ("nbtb = %d\n",nbtb); fflush (stdout);
	return nbtb;
}
/*
	Obtient la liste des sous-rpertoires dans un rpertoire virtuel
	retourne -1 si erreur, le nombre de sous-rpertoires placs dans tb[].
	Les sous-rpertoires sont placs via strdup_err(). L'appelant est
	responsable de librer.
*/
int filedir_getlistd (
	FILEDIR *fdir,
	const char *wild,		/* Wild card sans path */
	char *tb[],
	int maxtb,				/* Nombre maximum d'item dans la table */
	const char *addpath)	/* path a inserer devant chaque fichiers */
{
	int nbtb = filedir_match (fdir,wild,tb,maxtb,addpath,1);
	char *tbreel[400];
	int nbreel = dir_getlistd (addpath,wild,0,tbreel,1000);
	return vdir_fusion (tb,nbtb,tbreel,nbreel);
}


#ifdef TEST

unsigned int _stklen=10000;

int main (int argc, char *argv[])
{
	int i;
	FILEDIR fdir;
	int nb;
	char *tb[200];
	filedir_init (&fdir,"e:/kit/build/_1_0/adm/makefile.dat",NULL);
	nb = filedir_getlist (&fdir,"*.*",tb,200,"");
	for (i=0; i<nb;  i++){
		printf ("%2d - %s\n",i+1,tb[i]);
	}
	nb = filedir_getlist (&fdir,"*.cfg",tb,200,"../commun");
	for (i=0; i<nb;  i++){
		printf ("%2d - %s\n",i+1,tb[i]);
	}
	nb = filedir_getlistd (&fdir,"*.*",tb,200,"");
	for (i=0; i<nb;  i++){
		printf ("%2d - %s\n",i+1,tb[i]);
	}
	nb = filedir_getlistd (&fdir,"*.cfg",tb,200,"../commun");
	for (i=0; i<nb;  i++){
		printf ("%2d - %s\n",i+1,tb[i]);
	}
	tbstr_free (tb,nb);
}

#endif

==> linuxconf-tools-1.7.src.tar.gz: linuxconf-tools-1.7/vfs/filedir1.cc <==
#include <stdlib.h>
#include <string.h>
#include "vfs.h"

static int filedir_match (
	FILEDIR *fdir,
	const char *wild,	/* Wild card sans path */
	char *tb[],
	int maxtb,			/* Nombre maximum d'item dans la table */
	const char *addpath,/* Path a inserer devant chaque nom de fichier */
	int matchdir)		/* Recherche les sous-rpertoires */
{
	int nbtb = 0;
	//printf ("avant setup\n"); fflush (stdout);
	//atoi("1"); fflush (stdout);
	filedir_setup (fdir);
	//printf ("apres setup\n"); fflush (stdout);
	{
		WILD w;
		VFILE_SPEC *vf = fdir->incl_files;
		int nb = fdir->fincl_nb;
		int i;
		wild_set (wild,&w);
		for (i=0; i<nb; i++, vf++){
			const char *fname = vf->fname;
			// Le makefile.dat est toujours cach, voir spec plus loin.
			if (strcmp(fname,"makefile.dat")!=0){
				/*
					Certaines entres d'un makefile.dat dcrivent des
					lments d'un sous-rpertoire. On doit les ignors.
				*/
				const char *ptsep = strchr(fname,'/');
				int selectd;
				char fname_tmp[PATH_MAX];
				if (ptsep == NULL) ptsep = strchr(fname,'\\');
				selectd = (ptsep != NULL) + matchdir;
				if (selectd == 2){
					char tmp = *ptsep;
					*(char*)ptsep = '\0';
					strcpy (fname_tmp,fname);
					*(char*)ptsep = tmp;
					fname = fname_tmp;
				}
				if (selectd != 1 && wild_test(fname,&w)){
					if (nbtb < maxtb){
						/*
							Les fonctions dir_getlist_ retourne des noms
							de fichier avec le path original inclus.
						*/
						char fpath[PATH_MAX];
						if (addpath[0] != '\0'){
							path_make (addpath,fname,fpath);
							fname = fpath;
						}
						tb[nbtb++] = strdup (fname);
					}
				}
			}
		}
	}
	return nbtb;
}

#if 0
/*
	fusionne tbreel dans tb en evitant les rptitions.
	Les deux tables sont tries.

	Retourne le nombre d'item dans tb[]. Les lments de tbreel[] sont
	librs.
*/
static int vdir_fusion (
	char *tb[],
	int nbtb,
	char *tbreel[],
	int nbreel)
{
	int notb = 0;
	int i;
	for (i=0; i<nbreel; i++){
		char *fname = tbreel[i];
		if (strcmp(fname,"makefile.dat")!=0
			&& strcmp(fname,"makefile.del")!=0){
			/* #Spcification: vdir / fichiers de controle / exclusion
				Les fichiers de controle makefile.dat et makefile.del
				sont cachs au fonction usager qui balaye des
				rpertoire: vdir_getlist.. 
			*/
			// makefile.dat est exclu lors du balayage du makefile.dat
			// lui-mme.
			int ajout = 1;		/* Ajoute  la fin de la liste */
			for ( ; notb < nbtb; notb++){
				int cmp = strcmp(fname,tb[notb]);
				if(cmp == 0){
					/* Dj l */
					ajout = 0;
					break;	
				}else if (cmp < 0){
					/* Pas l, on ajoute */
					tbstr_insert (tb,nbtb,fname,notb,0);
					fname = NULL;
					nbtb++;
					notb++;
					ajout = 0;
					break;
				}
			}
			if (ajout){
				tb[nbtb++] = fname;
				fname = NULL;
				notb++;
			}
		}
		free (fname);
	}
	return nbtb;
}
#endif
/*
	Obtient la liste des des fichiers dans un rpertoire virtuel
	retourne -1 si erreur, le nombre de fichiers placs dans tb[].
	Les fichiers sont placs via strdup_err(). L'appelant est responsable
	de librer.
*/
int filedir_getlist (
	FILEDIR *fdir,
	const char *wild,		/* Wild card sans path */
	char *tb[],
	int maxtb,				/* Nombre maximum d'item dans la table */
	const char *addpath)	/* path a inserer devant chaque fichiers */
{
	//printf ("avant match :%s: :%s: %lx %p\n",wild,addpath,_ESP,filedir_match); fflush (stdout);
	//int nbtb = filedir_match (fdir,wild,tb,maxtb,addpath,0);
	int nbtb = filedir_match (fdir,wild,tb,maxtb,"",0);
	#if 0
		char *tbreel[1000];
		//printf ("Avant dir_getlist :%s: :%s:\n",addpath,wild);
		int nbreel = dir_getlist (addpath,wild,0,tbreel,1000);
		//printf ("nbreel = %d\n",nbreel); fflush (stdout);
		nbtb = vdir_fusion (tb,nbtb,tbreel,nbreel);
		//printf ("nbtb = %d\n",nbtb); fflush (stdout);
	#endif
	return nbtb;
}
#if 0
/*
	Obtient la liste des sous-rpertoires dans un rpertoire virtuel
	retourne -1 si erreur, le nombre de sous-rpertoires placs dans tb[].
	Les sous-rpertoires sont placs via strdup_err(). L'appelant est
	responsable de librer.
*/
int filedir_getlistd (
	FILEDIR *fdir,
	const char *wild,		/* Wild card sans path */
	char *tb[],
	int maxtb,				/* Nombre maximum d'item dans la table */
	const char *addpath)	/* path a inserer devant chaque fichiers */
{
	int nbtb = filedir_match (fdir,wild,tb,maxtb,addpath,1);
	char *tbreel[400];
	int nbreel = dir_getlistd (addpath,wild,0,tbreel,1000);
	return vdir_fusion (tb,nbtb,tbreel,nbreel);
}

#endif

#ifdef TEST

unsigned int _stklen=10000;

int main (int argc, char *argv[])
{
	int i;
	FILEDIR fdir;
	int nb;
	char *tb[200];
	filedir_init (&fdir,"e:/kit/build/_1_0/adm/makefile.dat",NULL);
	nb = filedir_getlist (&fdir,"*.*",tb,200,"");
	for (i=0; i<nb;  i++){
		printf ("%2d - %s\n",i+1,tb[i]);
	}
	nb = filedir_getlist (&fdir,"*.cfg",tb,200,"../commun");
	for (i=0; i<nb;  i++){
		printf ("%2d - %s\n",i+1,tb[i]);
	}
	nb = filedir_getlistd (&fdir,"*.*",tb,200,"");
	for (i=0; i<nb;  i++){
		printf ("%2d - %s\n",i+1,tb[i]);
	}
	nb = filedir_getlistd (&fdir,"*.cfg",tb,200,"../commun");
	for (i=0; i<nb;  i++){
		printf ("%2d - %s\n",i+1,tb[i]);
	}
	tbstr_free (tb,nb);
}

#endif
